'''
This is the fastest rollback implementation,
this script shouldn't be here and should be as standalone tool -
implemented like that due to lack of time :(
'''

import os
import subprocess

from . import rhel6_profile, snapshot_metadata_file, all_kernels_file, active_kernel_file
from .snapshot import LVM
from .bootloader import boom_cleanup, restore_boot


if __name__ == '__main__':
    try:
        lvm = LVM(conf_path=snapshot_metadata_file)
        lvm.restore_snapshots()
    except Exception:
        print "Error: unable to restore snapshots"
        raise SystemExit(1)

    # TODO: add boom binary to RHEL7
    # boom_cleanup(rhel6_profile)
    try:
        with open(active_kernel_file) as f_active_kernel:
            active_kernel = f_active_kernel.read()
            restore_boot(active_kernel)

            with open(all_kernels_file) as f_all_kernels:
                all_kernels = f_all_kernels.read()
                all_kernels = all_kernels.split('\n')
                for kernel in all_kernels:
                    if active_kernel in kernel:
                        continue
                    subprocess.call(["grubby", "--grub", "--remove-kernel", kernel.replace('kernel', '/boot/vmlinuz')])
    except Exception:
        print "Error: unable to restore boot config"
        raise SystemExit(1)

    os.system("reboot")



#!/usr/bin/python

import json
import os
import re
import shlex
import subprocess
import ConfigParser
import shutil

try:
    from redhat_upgrade_tool.rollback import rollback_dir, snap_boot_files_file
except ImportError:
    from . import rollback_dir, snap_boot_files_file

Config = ConfigParser.ConfigParser()
Config.read("/boot/grub/snapshot.metadata")


def run_subprocess(cmd, print_output=True):
    """
    Call the passed command and optionally print its output.
    """
    cmd = shlex.split(cmd, False)
    sp_popen = subprocess.Popen(cmd,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT,
                                bufsize=1)
    stdout = ''
    for line in iter(sp_popen.stdout.readline, ''):
        stdout += line
    sp_popen.communicate()

    return stdout, sp_popen.returncode


def load_json(input_file):
    with open(input_file, 'r') as f:
        return json.load(f)


def get_config_sections(section):
    dict1 = {}
    options = Config.options(section)
    for option in options:
        try:
            dict1[option] = Config.get(section, option)
        except:
            dict1[option] = None
    return dict1


def get_snapshot_path():
    snapshot_paths = []
    for part in Config.sections():
        name = get_config_sections(part)['name']
        origin_lv = get_config_sections(part)['origin_lv']
        vg_name, lv_name = origin_lv.split('/')
        lv_name = name
        snapshot_path = os.path.join('/dev', vg_name, lv_name)
        snapshot_paths.append(snapshot_path)
    return snapshot_paths


def remove_snapshot():
    for snapshot in get_snapshot_path():
        if os.path.exists(snapshot):
            print "Removing {} snapshot".format(snapshot)
            subprocess.call(['lvremove', '-f', snapshot])


def remove_snap_boot_files():
    snap_boot_files_data = load_json(snap_boot_files_file)
    for snap_boot_file in snap_boot_files_data:
        if os.path.isfile(snap_boot_file):
            print snap_boot_file, " will be deleted"
            os.remove(snap_boot_file)


def remove_loader_cache():
    directory = '/boot/loader/entries'
    for filename in os.listdir(directory):
        filename = os.path.join(directory, filename)
        if os.path.isfile(filename):
            print "Deleting ", filename
            os.remove(filename)


def clean_grub_entry():
    return run_subprocess('grubby --grub --remove-kernel=/boot/vmlinuz-snapshot')

def clean_rut_boot_dirs():
    shutil.rmtree(rollback_dir, ignore_errors=True)

if __name__ == "__main__":
    remove_snap_boot_files()
    remove_snapshot()
    remove_loader_cache()
    clean_grub_entry()
    # do this in the end to not keep the mess on the boot partition
    clean_rut_boot_dirs()



from collections import namedtuple
from mock import MagicMock
from redhat_upgrade_tool import sysprep


def test_setup_upgradelink():
    """ setup_upgradelink will set up the symlink between packagedir and upgradelink """
    sysprep.os.symlink = MagicMock(return_value=True)
    sysprep.setup_upgradelink()
    sysprep.os.symlink.assert_called_once_with('/var/lib/system-upgrade', '/system-upgrade')


def test_setup_upgraderoot():
    """ setup_upgraderoot will set up upgraderoot path """
    sysprep.os.path.isdir = MagicMock(return_value=False)
    sysprep.os.makedirs = MagicMock(return_value=True)
    sysprep.setup_upgraderoot()
    sysprep.os.makedirs.assert_called_once_with('/system-upgrade-root', 0755)



#!/usr/bin/python
#
# redhat-upgrade-tool.py - commandline frontend for redhat-upgrade-tool, the
# Red Hat Upgrade Tool.
#
# Copyright (C) 2012 Red Hat Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Will Woods <wwoods@redhat.com>

import os
import re
import shlex
import sys, time, platform, shutil, signal
from subprocess import CalledProcessError, Popen, PIPE
from ConfigParser import NoOptionError, RawConfigParser

from redhat_upgrade_tool.util import call, check_call, check_output, rm_f, mkdir_p, rlistdir, kernelver
from redhat_upgrade_tool.download import UpgradeDownloader, YumBaseError, yum_plugin_for_exc, URLGrabError
from redhat_upgrade_tool.sysprep import prep_upgrade, prep_boot, setup_media_mount, setup_cleanup_post, disable_old_repos, Config
from redhat_upgrade_tool.sysprep import modify_repos, remove_cache, reset_boot
from redhat_upgrade_tool.boot import upgrade_boot_args
from redhat_upgrade_tool.rollback import snapshot_metadata_file, rhel6_profile
from redhat_upgrade_tool.rollback.bootloader import boom_cleanup, restore_boot, create_boot_entry, restore_grub_conf, backup_boot_files, change_boot_entry, clean_snapshot_boot_files, clean_target_boot_files, clean_grub2, clean_target_kdump
from redhat_upgrade_tool.rollback.snapshot import LVM, SnapshotError
from redhat_upgrade_tool.rollback.preparecleanup import create_cleanup_script, dump_target_kernelver
from redhat_upgrade_tool.rollback.cleanup_script import clean_rut_boot_dirs
from redhat_upgrade_tool.upgrade import RPMUpgrade, TransactionError

from redhat_upgrade_tool.commandline import parse_args, do_cleanup, device_setup
from redhat_upgrade_tool import textoutput as output
from redhat_upgrade_tool import upgradeconf
from redhat_upgrade_tool import rhel_gpgkey_path
from redhat_upgrade_tool import preupgrade_script_path
from redhat_upgrade_tool import release_version_file
from redhat_upgrade_tool import _, kernelpath, initrdpath
from redhat_upgrade_tool import grub_conf_file
from redhat_upgrade_tool import MIN_AVAIL_BYTES_FOR_BOOT

import redhat_upgrade_tool.logutils as logutils
import redhat_upgrade_tool.media as media

from preupg.xccdf import XccdfHelper
from preupg import settings

import logging
log = logging.getLogger("redhat-upgrade-tool")


def message(m):
    print m
    log.info(m)


def setup_downloader(version, instrepo=None, cacheonly=False, repos=[],
                     enable_plugins=[], disable_plugins=[], noverifyssl=False):
    log.debug("setup_downloader(version=%s, repos=%s)", version, repos)
    f = UpgradeDownloader(version=version, cacheonly=cacheonly)
    f.preconf.enabled_plugins += enable_plugins
    f.preconf.disabled_plugins += disable_plugins
    f.instrepoid = instrepo
    repo_cb = output.RepoCallback()
    repo_prog = output.RepoProgress(fo=sys.stderr)
    disabled_repos = f.setup_repos(callback=repo_cb,
                                   progressbar=repo_prog,
                                   repos=repos,
                                   noverifyssl=noverifyssl)
    disabled_repos = filter(lambda id: id != f.instrepoid, disabled_repos)
    if disabled_repos:
        print _("No upgrade available for the following repos") + ": " + \
                " ".join(disabled_repos)
        print _("Check that the repo URLs are correct.")
        log.info("disabled repos: " + " ".join(disabled_repos))
    return f


def download_packages(f):
    updates = f.build_update_transaction(callback=output.DepsolveCallback(f))
    # check for empty upgrade transaction
    if not updates:
        print _('No upgrade found, please check the repository specified is correct.')
        print _('Finished. Nothing to do.')
        raise SystemExit(0)
    # print dependency problems before we start the upgrade
    transprobs = f.describe_transaction_problems()
    if transprobs and not major_upgrade:
        print "WARNING: potential problems with upgrade"
        for p in transprobs:
            print "  " + p
    # clean out any unneeded packages from the cache
    f.clean_cache(keepfiles=(p.localPkg() for p in updates))
    # download packages
    f.download_packages(updates, callback=output.DownloadCallback())

    return updates


def transaction_test(pkgs):
    print _("testing upgrade transaction")
    pkgfiles = set(po.localPkg() for po in pkgs)
    fu = RPMUpgrade()
    probs = fu.setup_transaction(pkgfiles=pkgfiles, check_fatal=False)
    rv = fu.test_transaction(callback=output.TransactionCallback(numpkgs=len(pkgfiles)))
    return (probs, rv)


def reboot():
    call(['reboot'])


def get_preupgrade_result_name():
    return os.path.join(settings.assessment_results_dir,
                        settings.xml_result_name)


def check_preupg_target_system_version(treeinfo):
    if not os.path.exists(release_version_file):
        print _("First, run the Preupgrade Assistant to analyze the system.")
        raise SystemExit(1)
    try:
        with open(release_version_file) as release_file:
            release = release_file.readlines()
    except (IOError, OSError) as e:
        print _("Unable to read the %s file provided by the Preupgrade"
                " Assistant:\n %s") % (release_version_file, e)
        raise SystemExit(1)
    try:
        preupg_supported_sysver = release[1].strip()
    except KeyError:
        print _("Error: The %s file provided by the Preupgrade Assistant has"
                " incorrect content.") % release_version_file
        raise SystemExit(1)
    installation_media_sysver = treeinfo.get('general', 'version')
    if preupg_supported_sysver != installation_media_sysver:
        print _("The installed version of Preupgrade Assistant allows upgrade"
                " only to the system version %s." % preupg_supported_sysver)
        raise SystemExit(1)

def is_clean_safe(snapshots):
    '''
    Do not allow performing --clean and --clean-snapshots options when booted into RHEL 6 snapshot.
    '''
    if not snapshots:
        return True
    with open('/proc/cmdline') as cmdline:
        res = re.search("root=(\S*)", cmdline.read())
        if res is None:
            print _("Error: unable to locate kernel root arg")
            raise SystemExit(1)
        root = res.group(1)
        for snapshot in snapshots.values():
            if root == snapshot.full_path:
                return False
    return True


def main(args):
    global major_upgrade

    try:
        lvm = LVM(args.snapshot_root_lv, args.snapshot_lv, conf_path=snapshot_metadata_file)
    except SnapshotError as exc:
        print _(exc)
        raise SystemExit(1)

    if args.snapshot_root_lv:
        create_cleanup_script()

    if args.system_restore:
        # TODO: .... add checks, exceptions, ....
        lvm.restore_snapshots()
        boom_cleanup(rhel6_profile)
        restore_boot()

        if args.reboot:
            reboot()
        else:
            print _('Preparation for recovery finished.'
                    ' Reboot to recover original system.')
        return

    if args.clean_snapshots:
        if not is_clean_safe(lvm.snapshots):
            print _("Error: cannot process --clean-snapshots option from booted snapshot, to rollback system use --system-restore option")
            raise SystemExit(1)

        lvm.remove_snapshots()
        boom_cleanup(rhel6_profile)
        clean_snapshot_boot_files()
        restore_grub_conf()
        return

    if not lvm.create_snapshots():
        print _("Error: could not create snapshot(s).")
        raise SystemExit(1)

    root_snapshot = lvm.get_root_snapshot()
    if root_snapshot is not None and root_snapshot.exists:
        # back up boot files & grub.conf before we touch the grub
        backup_boot_files()

        if not create_boot_entry("RHEL 6 Snapshot", rhel6_profile, root_snapshot.lv):
            print _("Error: could not create a boot entry for the snapshot.")
            raise SystemExit(1)

        if not change_boot_entry():
            print _("Error: could not change boot entry created by boom.")
            raise SystemExit(1)

    if args.clean:
        if not is_clean_safe(lvm.snapshots):
            print _("Error: cannot process --clean option from snapshot, to rollback system use --system-restore option")
            raise SystemExit(1)

        do_cleanup(args)
        lvm.remove_snapshots()
        boom_cleanup(rhel6_profile)
        clean_snapshot_boot_files()
        clean_target_boot_files()
        clean_target_kdump()
        restore_grub_conf()
        clean_grub2()
        clean_rut_boot_dirs()
        return
    else:
        # Leaving cache from previous runs of the tool could foil the correct
        # download of packages for upgrade (bz#1303982)
        remove_cache()

    if args.device or args.iso:
        device_setup(args)

    # Get our packages set up where we can use 'em
    print _("setting up repos...")
    f = setup_downloader(version=args.network,
                         cacheonly=args.cacheonly,
                         instrepo=args.instrepo,
                         repos=args.repos,
                         enable_plugins=args.enable_plugins,
                         disable_plugins=args.disable_plugins,
                         noverifyssl=args.noverifyssl)

    if not args.force:
        check_preupg_target_system_version(f.treeinfo)

    if is_major_version_upgrade(f.treeinfo):
        major_upgrade = True
        if not args.force:
            check_preupg_risks()
        else:
            log.info("Skipping examining the Preupgrade Assistant results.")

    if not args.force:
        check_same_variant_upgrade(f.treeinfo)
    else:
        log.info("Skipping system variant check.")

    if args.nogpgcheck:
        f._override_sigchecks = True
    elif not f.instrepo.gpgcheck:
        # If instrepo is a Red Hat repo, add the gpg key and reload the repos
        try:
            key = "product"
            if not f.treeinfo.has_section(key):
                key = "release"
            if f.treeinfo.get(key, 'name') == 'Red Hat Enterprise Linux':
                log.info("Reloading repos with GPG key")
                args.repos.append(('gpgkey', '%s=%s' % (f.instrepo.name, rhel_gpgkey_path)))
                f = setup_downloader(version=args.network,
                                     cacheonly=args.cacheonly,
                                     instrepo=args.instrepo,
                                     repos=args.repos,
                                     enable_plugins=args.enable_plugins,
                                     disable_plugins=args.disable_plugins,
                                     noverifyssl=args.noverifyssl)
        except NoOptionError:
            log.debug("No product name found, skipping gpg check")

    if args.expire_cache:
        print "expiring cache files"
        f.cleanExpireCache()
        return
    if args.clean_metadata:
        print "cleaning metadata"
        f.cleanMetadata()
        return

    # Cleanup old conf files
    log.info("Clearing %s", upgradeconf)
    rm_f(upgradeconf)
    mkdir_p(os.path.dirname(upgradeconf))

    # TODO: error msg generation should be shared between CLI and GUI
    if args.skipkernel:
        message("skipping kernel/initrd download")
    elif f.instrepoid is None or f.instrepoid in f.disabled_repos:
        print _("Error: can't get boot images.")
        if args.instrepo:
            print _("The '%s' repo was rejected by yum as invalid.") % args.instrepo
            if args.iso:
                print _("The given ISO probably isn't an install DVD image.")
                media.umount(args.device.mnt)
            elif args.device:
                print _("The media doesn't contain a valid install DVD image.")
        else:
            print _("The installation repo isn't currently available.")
            print _("Try again later, or specify a repo using --instrepo.")
        raise SystemExit(1)
    else:
        print _("getting boot images...")
        kernel, initrd = f.download_boot_images() # TODO: force arch?
        if args.snapshot_root_lv:
            # In case of rollback, we want to be able to remove kernel files
            # of target (upgraded) system after the rollback and be sure that
            # we will not remove those that possibly exists
            kv = kernelver(kernel)
            if not kv:
                print _("Warning: cannot determine version of target kernel."
                        " In case of rollback, you will need to remove kernel"
                        " files of target kernel manually.")
            elif os.path.isfile("/boot/vmlinuz-%s" % kv):
                print _("Warning: detected kernel files matching with"
                        " kernel of target system. In case of rollback, you"
                        " would need to clean these files manually.")
            else:
                dump_target_kernelver(kv)

    if args.skippkgs:
        message("skipping package download")
    else:
        print _("setting up update...")
        if len(f.pkgSack) == 0:
            print("no updates available in configured repos!")
            raise SystemExit(1)
        pkgs = download_packages(f)
        # Run a test transaction
        probs, rv = transaction_test(pkgs)

    # And prepare for upgrade
    # TODO: use polkit to get root privs for these things
    print _("setting up system for upgrade")
    if not args.skippkgs:
        prep_upgrade(pkgs)

    # Disable the RHEL-6 repos
    disable_old_repos()

    # Save the repo configuration
    f.save_repo_configs()

    # Dump all configuration to upgrade.conf, other tools need to know
    #TODO:some items are structured, would be nice to unpack them
    with Config(upgradeconf) as conf:
        argsdict = args.__dict__
        for arg in argsdict:
            conf.set("config", arg.__str__(), argsdict[arg].__str__())

    if args.cleanup_post:
        setup_cleanup_post()

    # Workaround the redhat-upgrade-dracut upgrade-post hook order problem
    # Copy upgrade.conf to /root/preupgrade so that it won't be removed
    # before the postupgrade scripts are run.
    mkdir_p('/root/preupgrade')
    shutil.copyfile(upgradeconf, '/root/preupgrade/upgrade.conf')

    # Run the preuprade scripts if present
    if os.path.isdir(preupgrade_script_path):
        scripts = sorted(rlistdir(preupgrade_script_path))
        failed_scripts = {}
        for s in scripts:
            if os.access(s, os.X_OK):
                try:
                    check_call(s)
                except CalledProcessError as e:
                    failed_scripts[s] = e.returncode
        if failed_scripts:
            print("Following preupgrade script(s) failed:\n")
            for key, val in failed_scripts.iteritems():
                print("%s exited with status %d" % (key, val))
            print('exiting')
            sys.exit(1)

    if not args.skipbootloader:
        if args.skipkernel:
            print "warning: --skipkernel without --skipbootloader"
            print "using default paths: %s %s" % (kernelpath, initrdpath)
            kernel = kernelpath
            initrd = initrdpath
        upgrade_boot_args()
        prep_boot(kernel, initrd)

    # Check for available space in /boot/ needed for kernel and grub
    # installation during the upgrade.
    statvfs = os.statvfs("/boot")
    avail_bytes = statvfs.f_frsize * statvfs.f_bavail
    if not args.no_space_check and avail_bytes < MIN_AVAIL_BYTES_FOR_BOOT:
        reset_boot()
        additional_mib_needed = \
            (MIN_AVAIL_BYTES_FOR_BOOT - avail_bytes) / 2**20
        sys.stderr.write(_("Not enough space. /boot/ needs additional %d MiB"
                           ".\n") % additional_mib_needed)
        raise SystemExit(1)

    # Replace temporary media paths
    modify_repos(args)

    if args.device:
        setup_media_mount(args.device, args.iso)

    if args.iso:
        media.umount(args.device.mnt)

    if args.reboot:
        reboot()
    else:
        print _('Finished. Reboot to start upgrade.')

    # --- Here's where we summarize potential problems. ---

    # list packages without updates, if any
    missing = sorted(f.find_packages_without_updates(), key=lambda p:p.nevra)
    if missing and not major_upgrade:
        message(_('Packages without updates:'))
        for p in missing:
            message("  %s" % p)

    # warn if the "important" repos are disabled
    #if f.disabled_repos:
        # NOTE: I hate having a hardcoded list of Important Repos here.
        # This information should be provided by the system, somehow..
        #important = ("fedora", "updates")
        #if any(i in f.disabled_repos for i in important):
        #    msg = _("WARNING: Some important repos could not be contacted: %s")
        #else:
        #    msg = _("NOTE: Some repos could not be contacted: %s")
        #print msg % ", ".join(f.disabled_repos)
        #print _("If you start the upgrade now, packages from these repos will not be installed.")

    # warn about broken dependencies etc.
    # If this is a major version upgrade, the user has already been warned
    # about all of this from preupgrade-assistant, so skip the warning here
    if probs and not major_upgrade:
        print
        print _("WARNING: problems were encountered during transaction test:")
        for s in probs.summaries:
            print "  "+s.desc
            for line in s.format_details():
                print "    "+line
        print _("Continue with the upgrade at your own risk.")


def is_major_version_upgrade(treeinfo):
    """Compare the first part of the version number in the treeinfo with the
    first part of the version number of the system to determine if this is a
    major version upgrade.
    """
    return treeinfo.get('general', 'version').split('.')[0] != \
        platform.linux_distribution()[1].split('.')[0]


def check_preupg_risks():
    returncode = XccdfHelper.check_inplace_risk(get_preupgrade_result_name(), 0)
    if int(returncode) == 0:
        print _("The Preupgrade Assistant hasn't found any risks.\n"
                "Upgrade will continue.")
    elif int(returncode) == 1:
        print _("The Preupgrade Assistant has found upgrade risks.\n"
                " You can run 'preupg --riskcheck --verbose' to view"
                " these risks.\nAddressing high risk issues is"
                " mandatory before continuing with the upgrade.\n"
                "Ignoring these risks may result in a broken and/or"
                " unsupported upgrade.\nPlease backup your data.\n\n"
                "List of issues:")

        XccdfHelper.check_inplace_risk(get_preupgrade_result_name(), verbose=2)

        # Python 2.6 raises EOFError if raw_input receives a SIGWINCH.
        # Try to tell the difference between that and a real EOF.

        global sigwinch
        sigwinch = False
        def handle_sigwinch(signum, frame):
            global sigwinch
            sigwinch = True
        orig_handler = signal.signal(signal.SIGWINCH, handle_sigwinch)

        while True:
            try:
                sigwinch = False
                answer = raw_input(_("Continue with the upgrade [Y/N]? "))
                break
            except EOFError:
                if sigwinch:
                    # Not a real EOF, try again
                    print
                    continue
                else:
                    # Real EOF, exit
                    answer = ''
                    break
        signal.signal(signal.SIGWINCH, orig_handler)

        # TRANSLATORS: y for yes
        if answer.lower() != _('y'):
            raise SystemExit(1)
    elif int(returncode) == 2:
        print _("The Preupgrade Assistant has found EXTREME upgrade"
                " risks.\nRun preupg --riskcheck --verbose to view\n"
                " these risks.\nContinuing with this upgrade is not\n"
                " recommended - the system will be unsupported\n"
                " and most likely broken after the upgrade.")
        raise SystemExit(1)
    else:
        print _("The Preupgrade Assistant has not been run.\n"
                "To upgrade the system, run preupg first.")
        raise SystemExit(1)


def check_same_variant_upgrade(treeinfo):
    distro = platform.linux_distribution()[0]
    if not distro.startswith("Red Hat Enterprise Linux "):
        print _("Invalid distribution: %s") % distro
        raise SystemExit(1)

    from_variant = distro[len('Red Hat Enterprise Linux '):]
    try:
        to_variant = treeinfo.get('general', 'variant')
    except NoOptionError:
        print _("Upgrade repository is not a Red Hat Enterprise Linux repository")
        raise SystemExit(1)

    if from_variant != to_variant:
        print _("Upgrade requested from Red Hat Enterprise Linux %s to %s") % (from_variant, to_variant)
        print _("Upgrades between Red Hat Enterprise Linux variants is not supported.")
        raise SystemExit(1)


if __name__ == '__main__':
    args = parse_args()
    major_upgrade = False

    # TODO: use polkit to get privs for modifying bootloader stuff instead
    if os.getuid() != 0:
        print _("you must be root to run this program.")
        raise SystemExit(1)

    # set up logging
    if args.debuglog:
        logutils.debuglog(args.debuglog)
    logutils.consolelog(level=args.loglevel)
    log.info("%s starting at %s", sys.argv[0], time.asctime())

    try:
        exittype = "cleanly"
        main(args)
    except KeyboardInterrupt as e:
        print
        log.info("exiting on keyboard interrupt")
        message(_("Exiting on keyboard interrupt"))
        raise SystemExit(1)
    except (YumBaseError, URLGrabError) as e:
        print
        if hasattr(e, "value") and isinstance(e.value, list):
            err = e.value.pop(0)
            message(_("Downloading failed: %s") % err)
            for p in e.value:
                message("  %s" % p)
        else:
            message(_("Downloading failed: %s") % e)
        log.debug("Traceback (for debugging purposes):", exc_info=True)
        raise SystemExit(2)
    except TransactionError as e:
        print
        message(_("Upgrade test failed with the following problems:"))
        for s in e.summaries:
            message(s)
        log.debug("Detailed transaction problems:")
        for p in e.problems:
            log.debug(p)
        log.error(_("Upgrade test failed."))
        raise SystemExit(3)
    except Exception as e:
        pluginfile = yum_plugin_for_exc()
        if pluginfile:
            plugin, ext = os.path.splitext(os.path.basename(pluginfile))
            log.error(_("The '%s' yum plugin has crashed.") % plugin)
            log.error(_("Please report this problem to the plugin developers:"),
                      exc_info=True)
            raise SystemExit(1)
        log.info("Exception:", exc_info=True)
        exittype = "with unhandled exception"
        raise
    finally:
        log.info("%s exiting %s at %s", sys.argv[0], exittype, time.asctime())




import os

rhel6_profile = "98c3edb"
snapshot_metadata_file = "/boot/grub/snapshot.metadata"
rollback_dir = "/boot/rollback"
active_kernel_file = os.path.join(rollback_dir, '.active-kernel')
all_kernels_file = os.path.join(rollback_dir, '.all-kernels')
target_kernel_file = os.path.join(rollback_dir, '.target-kernel')
snap_boot_files_file = os.path.join(rollback_dir, '.snap_boot_files')
grub2_exists_file = os.path.join(rollback_dir, '.grub2_exists')



import os
from subprocess import CalledProcessError, call
from ConfigParser import RawConfigParser, DuplicateSectionError

try:
    from redhat_upgrade_tool.util import check_call
except ImportError:
    def check_call(*popenargs, **kwargs):
        retcode = call(*popenargs, **kwargs)
        if retcode:
            cmd = kwargs.get("args")
            if cmd is None:
                cmd = popenargs[0]
            raise CalledProcessError(retcode, cmd)
        return 0


# this is not Red Hat Upgrade tool style
# but we have to fix exception handling
# we should avoid raising SystemExit(1) everywhere
class BaseSnapshotError(Exception):
    pass


class SnapshotError(BaseSnapshotError):
    pass


class SnapshotMetaConfig(object):

    def __init__(self, path):
        self.path = path
        self._config = RawConfigParser()
        self._config.read(path)

    def list(self):
        return [(
            self._config.get(section, "origin_lv"),
            self._config.get(section, "name"),
            self._config.get(section, "size")
        ) for section in self._config.sections()]

    def save_all(self, snapshots):
        for snapshot in snapshots:
            if not isinstance(snapshot, Snapshot):
                raise TypeError("Invalid snapshot type!")
            try:
                self._config.add_section(snapshot.lv)
            except DuplicateSectionError:
                # allow to override section
                pass
            for param in ("origin_lv", "name", "size"):
                self._config.set(snapshot.lv, param, getattr(snapshot, param))
            with open(self.path, "wb") as meta_file:
                self._config.write(meta_file)

    def remove_all(self, sections=None):
        if sections is None:
            sections = self._config.sections()
        for section in sections:
            self._config.remove_section(section)
        with open(self.path, "wb") as meta_file:
            self._config.write(meta_file)


class LVM(object):
    snapshots = {}

    def __init__(self, root_snap_args=None, snap_args=None, conf_path=None):
        root_snap_args = [] if root_snap_args is None else root_snap_args
        snap_args = [] if snap_args is None else snap_args

        sections_to_remove = []
        self.metadata_conf = SnapshotMetaConfig(conf_path)

        for origin_lv, name, size in self.metadata_conf.list():
            snapshot = Snapshot(origin_lv, name, size, exists=True)
            if os.path.exists(snapshot.full_path):
                self._add_snapshot(snapshot)
            else:
                sections_to_remove.append(snapshot.lv)
        # if there is snapshot section without existing snapshot then we have to remove it from file
        if sections_to_remove:
            self.metadata_conf.remove_all(sections_to_remove)

        root_lv = None
        if root_snap_args:
            root_lv, _, _ = self.get_snapshot_opt(root_snap_args)

        for args in list(snap_args):
            origin_lv, name, size = self.get_snapshot_opt(args)
            is_root = True if root_lv == origin_lv else False
            self._add_snapshot(Snapshot(origin_lv, name, size, is_root))

    def _add_snapshot(self, obj):
        lv = repr(obj)
        if lv not in self.snapshots:
            self.snapshots[lv] = obj
        # it's possible to indicate new root partition
        # obj == self.snapshots[lv] is checking only origin_lv, name and size without root
        elif lv in self.snapshots and obj == self.snapshots[lv]:
            obj.exists = True
            self.snapshots[lv] = obj
        else:
            raise SnapshotError(
                "Snapshot %s already exists with different parameters, run --clean-snapshots first" % lv
            )

    @property
    def snapshots_len(self):
        return len(self.snapshots)

    @staticmethod
    def get_snapshot_opt(opt):
        if isinstance(opt, (list, tuple)) and len(opt) == 3:
            origin_lv, name, size = opt
            return (origin_lv, name, size)
        raise ValueError("Invalid snapshot params len!")

    # TODO: this function has to be more generic
    # we need to have possibility to get snapshot by
    # path, name
    def get_root_snapshot(self):
        for snapshot in self.snapshots.values():
            if snapshot.root:
                return snapshot

    def create_snapshots(self):
        for snapshot in self.snapshots.values():
            if not snapshot.create():
                self.remove_snapshots()
                return False
        self.metadata_conf.save_all(self.snapshots.values())
        return True

    def remove_snapshots(self):
        for snapshot in self.snapshots.values():
            if not snapshot.remove():
                return False
        self.metadata_conf.remove_all()
        return True

    def restore_snapshots(self):
        for snapshot in self.snapshots.values():
            snapshot.merge()
        self.metadata_conf.remove_all()


class Snapshot(object):

    def __init__(self, origin_lv, name=None, size=None, root=False, exists=False):
        self.origin_lv = origin_lv
        self.name = name
        self.size = size
        self.root = root
        self.exists = exists

    def __repr__(self):
        return self.lv

    def __eq__(self, other):
        if not isinstance(other, Snapshot):
            return False
        return (self.origin_lv, self.name, self.size) == \
                (other.origin_lv, other.name, other.size)

    @property
    def lv(self):
        return os.path.join(os.path.split(self.origin_lv)[0], self.name)

    @property
    def full_path(self):
        if self.lv.startswith('/dev'):
            return self.lv
        return os.path.join('/dev', self.lv)

    def create(self):
        if self.exists:
            return True

        size_opt, size = ("-l", "100%ORIGIN") if not self.size else ("--size", self.size)
        cmd = [
            "lvcreate",
            size_opt, size,
            "--snapshot",
            "--name", self.name,
            self.origin_lv
        ]
        try:
            check_call(cmd)
        except CalledProcessError:
            return False
        else:
            self.exists = True
            return True

    def remove(self):
        if not self.exists:
            return True

        cmd = ["lvremove", "-f", self.lv]
        try:
            check_call(cmd)
        except CalledProcessError:
            return False
        else:
            self.exists = False
            return True

    def merge(self):
        if not self.exists:
            return False

        cmd = ["lvconvert", "--merge", self.lv]
        try:
            check_call(cmd)
        except CalledProcessError:
            return False
        else:
            self.exists = False
            return True



import os
from mock import MagicMock
from redhat_upgrade_tool import util


def test_listdir():
    """ listdir returns a list of directory content """
    util.os.listdir = MagicMock(return_value=['file1.txt', 'file2.txt', 'file3.txt'])
    actual = []
    for entry in util.listdir('mockdir'):
        actual.append(entry)
    assert actual == ['mockdir/file1.txt', 'mockdir/file2.txt', 'mockdir/file3.txt']


def test_mkdir_p():
    """ mkdir_p create directories """
    util.os.makedirs = MagicMock(return_value=True)
    util.mkdir_p('/some/mock/path')
    util.os.makedirs.assert_called_once_with('/some/mock/path')


def test_rm_f():
    """ rm_f deletes some file """
    util.os.path.lexists = MagicMock(return_value=True)
    util.os.remove = MagicMock(return_value=True)
    util.rm_f('/some/mock/path', util.os.remove)
    util.os.path.lexists.assert_called_once_with('/some/mock/path')
    util.os.remove.assert_called_once_with('/some/mock/path')


def test_kernelver():
    """ kernelver returns a version from a vmlinuz file """
    basedir = os.path.dirname(os.path.abspath(__file__))
    data_path = os.path.join(basedir, 'data')

    kernel_file = os.path.join(data_path, 'vmlinuz-4.12.9-300.fc26.x86_64')
    assert util.kernelver(kernel_file) == '4.12.9-300.fc26.x86_64'


def test_hrsize():
    """ hrsize converts size to a human readable format """

    data = [
        {'size': 0, 'si': False, 'use_ib': False, 'expected': '1K'},
        {'size': 0, 'si': False, 'use_ib': True, 'expected': '1KiB'},
        {'size': 0, 'si': True, 'use_ib': False, 'expected': '1KB'},
        {'size': 0, 'si': True, 'use_ib': True, 'expected': '1KB'},

        {'size': 42, 'si': False, 'use_ib': False, 'expected': '1K'},
        {'size': 42, 'si': False, 'use_ib': True, 'expected': '1KiB'},
        {'size': 42, 'si': True, 'use_ib': False, 'expected': '1KB'},
        {'size': 42, 'si': True, 'use_ib': True, 'expected': '1KB'},

        {'size': 512, 'si': False, 'use_ib': False, 'expected': '1K'},
        {'size': 512, 'si': False, 'use_ib': True, 'expected': '1KiB'},
        {'size': 512, 'si': True, 'use_ib': False, 'expected': '1KB'},
        {'size': 512, 'si': True, 'use_ib': True, 'expected': '1KB'},

        {'size': 1024, 'si': False, 'use_ib': False, 'expected': '2K'},
        {'size': 1024, 'si': False, 'use_ib': True, 'expected': '2KiB'},
        {'size': 1024, 'si': True, 'use_ib': False, 'expected': '2KB'},
        {'size': 1024, 'si': True, 'use_ib': True, 'expected': '2KB'},

        {'size': 1024 * 1024, 'si': False, 'use_ib': False, 'expected': '2M'},
        {'size': 1024 * 1024, 'si': False, 'use_ib': True, 'expected': '2MiB'},
        {'size': 1024 * 1024, 'si': True, 'use_ib': False, 'expected': '2MB'},
        {'size': 1024 * 1024, 'si': True, 'use_ib': True, 'expected': '2MB'},
    ]

    for entry in data:
        actual = util.hrsize(entry['size'], entry['si'], entry['use_ib'])
        assert actual == entry['expected']



# preparecleanup.py
#
# Copyright (C) 2018 Red Hat Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Jakub Mazanek <jmazanek@redhat.com>

import json
import os
import shutil
import subprocess
import platform
from redhat_upgrade_tool.util import mkdir_p
from redhat_upgrade_tool.rollback import rollback_dir, snap_boot_files_file, grub2_exists_file
from redhat_upgrade_tool.rollback import active_kernel_file, all_kernels_file, target_kernel_file
from redhat_upgrade_tool.rollback.bootloader import _SNAP_BOOT_FILES


def create_cleanup_script():
    rollback_path = os.path.dirname(os.path.abspath(__file__))
    if os.path.exists(rollback_dir):
        shutil.rmtree(rollback_dir)
    shutil.copytree(rollback_path, rollback_dir)

    with open(active_kernel_file, 'w') as active_kernel:
        active_kernel.write(platform.release())

    with open(all_kernels_file, 'w') as all_kernels:
        all_kernels.write(subprocess.Popen(["rpm", "-qa", "kernel"], stdout=subprocess.PIPE).communicate()[0])

    script_path = os.path.join(rollback_dir, 'do_rollback')
    with open(script_path, 'wb') as script_file:
        script_file.write("#!/bin/bash\ncd /boot && python -m rollback.system_restore\n")
    os.chmod(script_path, 0o774)

    script_path = os.path.join(rollback_dir, 'do_cleanup')
    with open(script_path, 'wb') as script_file:
        script_file.write("#!/bin/bash\ncd /boot && python -m rollback.cleanup_script\n")
    os.chmod(script_path, 0o774)

    dump_snapshot_boot_files()
    dump_grub2_exists()


def dump_target_kernelver(kv):
    # kv = kernel version
    with open(target_kernel_file, 'w') as target_kernel:
        target_kernel.write(kv)

def dump_grub2_exists():
    if os.path.isdir("/boot/grub2"):
        with open(grub2_exists_file, 'w') as f:
            f.write("1")

def dump_snapshot_boot_files():
    _SNAP_BOOT_PATHS = []
    for fmt in _SNAP_BOOT_FILES:
        path = os.path.join("/boot", fmt.format("snapshot"))
        _SNAP_BOOT_PATHS.append(path)
        dump_vars(_SNAP_BOOT_PATHS, snap_boot_files_file)


def dump_vars(variables, out_file):
    with open(out_file, 'w+') as outfile:
        json.dump(variables, outfile)



#!/usr/bin/env python

"""
Simple PoC script to restore a snapshot taken with RUT.
"""

import os
import re
import sys
import shutil
import argparse
import platform
from subprocess import check_call


def rename_files():
    release = platform.release()
    formats = [
        "initramfs-{0}.img",
        "vmlinuz-{0}",
        "System.map-{0}",
        "symvers-{0}.gz",
        "config-{0}",
    ]
    for fmt in formats:
        src = os.path.join("/boot", fmt.format("snapshot"))
        dst = os.path.join("/boot", fmt.format(release))
        shutil.copy2(src, dst)


def change_boot_entry(snap_lv, root_lv):
    GRUB_CFG = "/boot/grub/menu.lst"
    with open(GRUB_CFG, "r") as fd:
        lines = fd.read()
    with open(GRUB_CFG, "a") as fd:
        pattern = r"#--- RUT_Grub1_BEGIN ---(.+?)#--- RUT_Grub1_END ---"
        regex = re.compile(pattern, re.DOTALL)
        entries = [x.strip() for x in regex.findall(lines) if x.strip()]
        if not entries:
            return False
        for entry in entries:
            final = "\n#--- RUT_Restored_Grub1_BEGIN ---\n"
            final += re.sub(r"snapshot", platform.release(), entry)
            final = re.sub(snap_lv, root_lv, final)
            final = final.replace("RHEL 6 Snapshot", "RHEL 6 Snapshot restored")
            final += "\n#--- RUT_Restored_Grub1_END ---\n"
            fd.write(final)
    return True


def restore_snapshot(snap_lv):
    cmd = [
        "lvconvert",
        "--merge", snap_lv
    ]
    check_call(cmd)


def errorf(msg):
    sys.stderr.write(msg+'\n')


def main(arguments):
    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    parser.add_argument('-r', '--root-lv', help="Root logical volume", type=str)
    parser.add_argument('-s', '--snapshot-lv', help="Snapshot volume", type=str)
    args = parser.parse_args(arguments)
    rename_files()
    if not change_boot_entry(args.snapshot_lv, args.root_lv):
        errorf("Could not change boot entry")
        return 1
    restore_snapshot(args.snapshot_lv)
    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



import os
import re
import shutil
import platform
from subprocess import CalledProcessError, call

try:
    from redhat_upgrade_tool.rollback import target_kernel_file, grub2_exists_file
except ImportError:
    from . import target_kernel_file, grub2_exists_file

try:
    from redhat_upgrade_tool import grub_conf_file
    from redhat_upgrade_tool.util import check_call
except ImportError:
    grub_conf_file = "/boot/grub/grub.conf"

    def check_call(*popenargs, **kwargs):
        retcode = call(*popenargs, **kwargs)
        if retcode:
            cmd = kwargs.get("args")
            if cmd is None:
                cmd = popenargs[0]
            raise CalledProcessError(retcode, cmd)
        return 0


_SNAP_BOOT_FILES = [
    "initramfs-{0}.img",
    "vmlinuz-{0}",
    "System.map-{0}",
    "symvers-{0}.gz",
    "config-{0}",
]

_BOOM_UTIL_PATH = "/usr/libexec/boom"


def create_boot_entry(title, os_profile, root_lv):
    cmd = [
        _BOOM_UTIL_PATH, "create",
        "--profile", os_profile,
        "--title", title,
        "--root-lv", root_lv
    ]
    try:
        check_call(cmd)
    except CalledProcessError:
        return False
    return True


def boom_cleanup(os_profile):
    cmd = [
        _BOOM_UTIL_PATH, "delete",
        "--profile", os_profile
    ]
    try:
        check_call(cmd)
    except CalledProcessError:
        return False
    return True


def backup_boot_files():
    release = platform.release()
    for fmt in _SNAP_BOOT_FILES:
        src = os.path.join("/boot", fmt.format(release))
        dst = os.path.join("/boot", fmt.format("snapshot"))
        shutil.copy2(src, dst)
    # back up grub config file
    shutil.copy2(grub_conf_file, "%s.preupg" % grub_conf_file)


def change_boot_entry():
    with open(grub_conf_file, "r") as fd:
        lines = fd.read()
    with open(grub_conf_file, "a") as fd:
        pattern = r"#--- BOOM_Grub1_BEGIN ---(.+?)#--- BOOM_Grub1_END ---"
        regex = re.compile(pattern, re.DOTALL)
        entries = [x.strip() for x in regex.findall(lines) if x.strip()]
        if not entries:
            return False
        fd.write("\ntitle RUT Snapshots\n")
        for entry in entries:
            fd.write("\n#--- RUT_Grub1_BEGIN ---\n")
            fd.write(re.sub(platform.release(), "snapshot", entry))
            fd.write("\n#--- RUT_Grub1_END ---\n")
    return True


def restore_boot(release=platform.release()):
    for fmt in _SNAP_BOOT_FILES:
        src = os.path.join("/boot", fmt.format("snapshot"))
        dst = os.path.join("/boot", fmt.format(release))
        shutil.move(src, dst)
    return restore_grub_conf()


def clean_snapshot_boot_files():
    for fmt in _SNAP_BOOT_FILES:
        path = os.path.join("/boot", fmt.format("snapshot"))
        if os.path.isfile(path):
            os.remove(path)


def clean_target_boot_files():
    # get target_kernel
    if not os.path.isfile(target_kernel_file):
        # nothing to do
        return
    with open(target_kernel_file, "rb") as kerf:
        target_kernel = kerf.read()
    for fmt in _SNAP_BOOT_FILES:
        path = os.path.join("/boot", fmt.format(target_kernel))
        if os.path.isfile(path):
            os.remove(path)


def clean_target_kdump():
    # After the rollback, kdump of target kernel could be made. Remove it
    # in case the target-kernel file exists.
    if not os.path.isfile(target_kernel_file):
        # nothing to do
        return
    with open(target_kernel_file, "rb") as kerf:
        target_kernel = kerf.read()
    path = "/boot/initramfs-{0}kdump.img".format(target_kernel)
    if os.path.isfile(path):
        os.remove(path)

def clean_grub2():
    # After rollback, grub2 dir is still present. Remove it when originally
    # didn't exist.
    if os.path.isfile(grub2_exists_file):
        shutil.rmtree("/boot/grub2", ignore_errors=True)

def restore_grub_conf():
    backup_file = "%s.preupg" % grub_conf_file
    if os.path.isfile(backup_file):
        shutil.move(backup_file, grub_conf_file)
        return True
    return False
